######################################################################
#
#  EPrints User Record Forms
#
######################################################################
#
# License for eprints.org software version: Version 1.1.2 (29/08/2001)
# 
# Copyright (C) 2001, University of Southampton
# 
# The University of Southampton retains the copyright of this software
# code with the exception of the open archives component (in the
# openarchives/ directory), which is a modified version of code
# distributed by Cornell University Digital Library Research Group.
# 
# This software is freely distributable. Modified versions of this
# software may be distributed provided that a file README is included
# describing the modifications and from where the original version may
# be obtained.
# 
# This software is provided with no guarantees of suitability for any
# intended purpose. Use of the software is entirely at the end user's
# risk.
#
######################################################################

package EPrints::UserForm;

use EPrints::User;
use EPrints::HTMLRender;
use EPrints::Session;
use EPrints::Database;

use strict;

######################################################################
#
# $userform = new( $session, $redirect, $staff, $user )
#
#  Create a new user form session. If $user is unspecified, the current
#  user (from Apache cookies) is used.
#
######################################################################

sub new
{
	my( $class, $session, $redirect, $staff, $user ) = @_;
	
	my $self = {};
	bless $self, $class;

	$self->{session} = $session;
	$self->{redirect} = $redirect;
	$self->{staff} = $staff;
	$self->{user} = $user;
	
	return( $self );
}


######################################################################
#
# process()
#
#  Render and respond to the form
#
######################################################################

sub process
{
	my( $self ) = @_;
	
	$self->{user} = EPrints::User::current_user( $self->{session} )
		unless( defined $self->{user} );

	if( !defined $self->{user} )
	{
		# Can't find the user
		$self->{session}->{render}->render_error( "I don't know who you are",
		                                  $self->{redirect} );
		return;
	}

	my $full_name = $self->{user}->full_name();

	if( $self->{session}->{render}->seen_form() == 0 ||
	    $self->{session}->{render}->internal_button_pressed() )
	{
		print $self->{session}->{render}->start_html( "Record for $full_name" );

		# Blurb
		print "<P>Please enter correct information about yourself for our ".
			"records. This	information will be useful to us and readers of your ".
			"papers. You don't have to	supply all this information if you don\'t ".
			"want to; you need only fill out those	fields marked with a * to ".
			"start using the archive.</P>\n";

		print "<P>For instructions on how to change your e-mail address, ".
			"<a href=\"$EPrintSite::SiteInfo::server_static/register.html\">".
			"click here</A>.</P>\n";

		$self->render_form();

		print $self->{session}->{render}->end_html();
	}
	else
	{
		# Update the user values
		if( $self->update_from_form() )
		{
			# Validate the changes
			my $problems = $self->{user}->validate();

			if( $#{$problems} == -1 )
			{
				# User has entered everything OK
				$self->{user}->commit();
				$self->{session}->{render}->redirect( $self->{redirect} );
			}
			else
			{
				print $self->{session}->{render}->start_html(
					"Record for $full_name" );

				print "<P>The form doesn\'t seem to be filled out correctly:</P>\n".
					"<UL>\n";

				foreach (@$problems)
				{
					print "<LI>$_</LI>\n";
				}

				print "</UL>\n<P>Please complete the form before continuing.</P>\n";

				$self->render_form();

				print $self->{session}->{render}->end_html();
			}
		}
		else
		{
			$self->{session}->{render}->render_error(
				"There was a problem reading the posted data and updating the ".
					"database. Please try again later",
				$self->{redirect} );
		}
	}
}


######################################################################
#
# render_form()
#
#  Render the current user as an HTML form for editing. If
# $self->{staff} is 1, the staff-only fields will be available for
#  editing, otherwise they won't.
#
######################################################################

sub render_form
{
	my( $self ) = @_;
	
	my @edit_fields;
	my $field;
	my @all_fields = EPrints::MetaInfo::get_user_fields();
	
	# Get the appropriate fields
	foreach $field (@all_fields)
	{
		push @edit_fields, $field if( $self->{staff} || $field->{editable} );
	}
	
	my %hidden = ( "username"=>$self->{user}->{username} );
	
	$self->{session}->{render}->render_form( \@edit_fields,
	                                         $self->{user},
	                                         1,
	                                         1,
	                                         [ "Update Record" ],
	                                         \%hidden );
}

######################################################################
#
# $success = update_from_form()
#
#  Updates the user object from POSTed form data. Note that this
#  methods does NOT update the database - for that use commit().
#
######################################################################

sub update_from_form
{
	my( $self ) = @_;
	
	my @all_fields = EPrints::MetaInfo::get_user_fields();
	my $field;

	# Ensure correct user
	if( $self->{session}->{render}->param( "username" ) eq
		$self->{user}->{username} )
	{
		foreach $field (@all_fields)
		{
			my $param = $self->{session}->{render}->form_value( $field );
			$param = undef if( defined $param && $param eq "" );

			# Only update if a value for the field was entered in the form.
			if( $self->{staff} || $field->{editable} )
			{
				$self->{user}->{$field->{name}} = $param;
			}
		}
		return( 1 );
	}
	else
	{
		my $form_id = $self->{session}->{render}->param( "username" );
		EPrints::Log::log_entry(
			"User",
			"Username in form $form_id doesn't match object username ".
				"$self->{username}" );

		return( 0 );
	}
}


1;
